--[[
   dynamically create an event catcher frame to do stuff after other addons are
   loaded
  ]]
local ShortTerm = LibStub("AceAddon-3.0"):NewAddon(
	"ShortTerm"
);

ShortTermMemory = {};
ShortTermMemoryEvents = {
	SYSTEM = true,         BATTLEGROUND_LEADER = true,
	SAY = true,            GUILD = true,
	EMOTE = true,          GUILD_OFFICER = true,   
	YELL = true,           MONSTER_SAY = true,  
	WHISPER = true,        MONSTER_YELL = true,  
	PARTY = true,          MONSTER_EMOTE = true, 
	RAID = true,           MONSTER_WHISPER = true,  
	RAID_LEADER = true,    MONSTER_BOSS_EMOTE = true,    
	RAID_WARNING = true,   MONSTER_BOSS_WHISPER = true,      
	BATTLEGROUND = true,   ERRORS = true,     
	AFK = true,            SKILL = true,
	DND = true,            LOOT = true,
	IGNORED = true,        MONEY = true,
	BG_HORDE = true,       OPENING = true,
	BG_ALLIANCE = true,    TRADESKILLS = true,
	BG_NEUTRAL = true,     PET_INFO = true,
	COMBAT_XP_GAIN = true, COMBAT_MISC_INFO = true,
	ACHIEVEMENT = true,    COMBAT_HONOR_GAIN = true,
	CHANNEL = true,        COMBAT_FACTION_CHANGE = true,
	GUILD_ACHIEVEMENT = true,
};


-- /spew LibStub("AceAddon-3.0"):GetAddon("ShortTerm").cache.data
local LibRingCache = LibStub:GetLibrary("LibRingCache-1.0");
local save = function(self, ...)
	--print(...);
	ShortTerm.cache:append({...});
end

local events = {
	SYSTEM                = { "CHAT_MSG_SYSTEM","TIME_PLAYED_MSG",
	                          "PLAYER_LEVEL_UP","CHARACTER_POINTS_CHANGED", },
	SAY                   = { "CHAT_MSG_SAY", },
	EMOTE                 = { "CHAT_MSG_EMOTE","CHAT_MSG_TEXT_EMOTE", },
	YELL                  = { "CHAT_MSG_YELL", },
	WHISPER               = { "CHAT_MSG_WHISPER","CHAT_MSG_WHISPER_INFORM",},
	PARTY                 = { "CHAT_MSG_PARTY","CHAT_MSG_MONSTER_PARTY", },
	RAID                  = { "CHAT_MSG_RAID", },
	RAID_LEADER           = { "CHAT_MSG_RAID_LEADER", },
	RAID_WARNING          = { "CHAT_MSG_RAID_WARNING", },
	BATTLEGROUND          = { "CHAT_MSG_BATTLEGROUND", },
	BATTLEGROUND_LEADER   = { "CHAT_MSG_BATTLEGROUND_LEADER", },
	GUILD                 = { "CHAT_MSG_GUILD","GUILD_MOTD", },
	GUILD_OFFICER         = { "CHAT_MSG_OFFICER", },
	MONSTER_SAY           = { "CHAT_MSG_MONSTER_SAY", },
	MONSTER_YELL          = { "CHAT_MSG_MONSTER_YELL", },
	MONSTER_EMOTE         = { "CHAT_MSG_MONSTER_EMOTE", },
	MONSTER_WHISPER       = { "CHAT_MSG_MONSTER_WHISPER", },
	MONSTER_BOSS_EMOTE    = { "CHAT_MSG_RAID_BOSS_EMOTE", },
	MONSTER_BOSS_WHISPER  = { "CHAT_MSG_RAID_BOSS_WHISPER", },
	ERRORS                = { "CHAT_MSG_RESTRICTED","CHAT_MSG_FILTERED", },
	AFK                   = { "CHAT_MSG_AFK", },
	DND                   = { "CHAT_MSG_DND", },
	IGNORED               = { "CHAT_MSG_IGNORED", },
	BG_HORDE              = { "CHAT_MSG_BG_SYSTEM_HORDE", },
	BG_ALLIANCE           = { "CHAT_MSG_BG_SYSTEM_ALLIANCE", },
	BG_NEUTRAL            = { "CHAT_MSG_BG_SYSTEM_NEUTRAL", },
	COMBAT_XP_GAIN        = { "CHAT_MSG_COMBAT_XP_GAIN", },
	COMBAT_HONOR_GAIN     = { "CHAT_MSG_COMBAT_HONOR_GAIN", },
	COMBAT_FACTION_CHANGE = { "CHAT_MSG_COMBAT_FACTION_CHANGE", },
	SKILL                 = { "CHAT_MSG_SKILL", },
	LOOT                  = { "CHAT_MSG_LOOT", },
	MONEY                 = { "CHAT_MSG_MONEY", },
	OPENING               = { "CHAT_MSG_OPENING", },
	TRADESKILLS           = { "CHAT_MSG_TRADESKILLS", },
	PET_INFO              = { "CHAT_MSG_PET_INFO", },
	COMBAT_MISC_INFO      = { "CHAT_MSG_COMBAT_MISC_INFO", },
	ACHIEVEMENT           = { "CHAT_MSG_ACHIEVEMENT", },
	GUILD_ACHIEVEMENT     = { "CHAT_MSG_GUILD_ACHIEVEMENT", },
	CHANNEL               = { "CHAT_MSG_CHANNEL_JOIN",   "CHAT_MSG_CHANNEL_LEAVE",
	                          "CHAT_MSG_CHANNEL_NOTICE", "CHAT_MSG_CHANNEL_NOTICE_USER",
	                          "CHAT_MSG_CHANNEL_LIST",   "CHAT_MSG_CHANNEL", },
};

function ShortTerm:OnEnable()
	-- set up ringbuffer
	self.cache = LibRingCache:create(30, ShortTermMemory);
	-- load old lines into the chat
	local f;
	local fs = {};
	for k,v in ipairs(ShortTermMemory) do
		for i=0, 9 do
			f = _G["ChatFrame"..i];
			if f and f:IsEventRegistered(v[1]) then
				ChatFrame_MessageEventHandler(_G["ChatFrame"..i], unpack(v));
				fs[f] = true;
			end
		end
	end
	for k in pairs(fs) do
		k:AddMessage("[|cFFFF5500End of short term memory|r]");
	end
	-- create a catcher frame to log messages
	local f = CreateFrame("frame", UIParent);
	for k,v in pairs(ShortTermMemoryEvents) do
		for l,event in pairs(events[k]) do
			f:RegisterEvent(event);
		end
	end
	f:SetScript("OnEvent", save);
end

function ShortTerm:OnDisable()
	return;
end
